#!/usr/bin/env python
# A Wizard to setup a simpleFoam case (for OpenFOAM 2.x) from a mesh produced in Salome (UNV format)
# J.Cugnoni, CAELinux.com, 2011
from Tkinter import *
from tkCommonDialog import *
from tkMessageBox import *
from tkFileDialog import *
import os
import os.path
import sys 

########################
# INITIALIZE VARIABLES 
########################

template_path='/opt/caelinux/Templates/simpleFoamTemplate'
env_path='/opt/.bashrc-OpenFOAM'

### TEMPLATES ###
# formats for the 0/U file
U_dict={}
U_dict['header']='''
/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     | Version:  2.0.1                                 |
|   \\  /    A nd           | Web:      www.OpenFOAM.com                      |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       volVectorField;
    object      U;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

dimensions      [0 1 -1 0 0 0 0];

internalField   uniform (0 0 0);

boundaryField
{
'''
U_dict['inlet']='%s\n{\n        type            fixedValue; \n        value           uniform (%f %f %f);\n}\n'
U_dict['outlet']='%s\n{\n       type            zeroGradient;\n}\n'
U_dict['wall']='%s\n{\n        type            fixedValue; \n        value           uniform (0. 0. 0.);\n}\n'
U_dict['symmetry']='%s\n{\n       type            empty;\n}\n'
U_dict['footer']='}\n'


# formats for the 0/p file
p_dict={}
p_dict['header']='''
/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     | Version:  2.0.1                                 |
|   \\  /    A nd           | Web:      www.OpenFOAM.com                      |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       volScalarField;
    object      p;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

dimensions      [0 2 -2 0 0 0 0];

internalField   uniform 0;

boundaryField
{
'''
p_dict['inlet']='%s\n{\n       type            zeroGradient;\n}\n'
p_dict['outlet']='%s\n{\n        type            fixedValue; \n        value           uniform 0;\n}\n'
p_dict['wall']='%s\n{\n        type            zeroGradient;\n}\n'
p_dict['symmetry']='%s\n{\n       type            empty;\n}\n'
p_dict['footer']='}\n'

# formats for the 0/R file
R_dict={}
R_dict['header']='''
/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     | Version:  2.0.1                                 |
|   \\  /    A nd           | Web:      www.OpenFOAM.com                      |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       volSymmTensorField;
    object      R;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

dimensions      [0 2 -2 0 0 0 0];

internalField   uniform (0 0 0 0 0 0);

boundaryField
{
'''
R_dict['inlet']='%s\n{\n type            fixedValue;\n        value           uniform (0 0 0 0 0 0);\n}\n'
R_dict['outlet']='%s\n{\n        type            zeroGradient;\n}\n'
R_dict['wall']='%s\n{\n        type            kqRWallFunction;\n        value           uniform ( 0 0 0 0 0 0 );\n}\n'
R_dict['symmetry']='%s\n{\n       type            empty;\n}\n'
R_dict['footer']='}\n'

# formats for the 0/k file
k_dict={}
k_dict['header']='''
/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     | Version:  2.0.1                                 |
|   \\  /    A nd           | Web:      www.OpenFOAM.com                      |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       volScalarField;
    location    "0";
    object      k;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

dimensions      [0 2 -2 0 0 0 0];

internalField   uniform %f;

boundaryField
{
'''  #  need initial value of k
k_dict['inlet']='%s\n{\n        type            fixedValue;\n        value           uniform %f;\n}\n'
k_dict['outlet']='%s\n{\n        type            zeroGradient;\n}\n'
k_dict['wall']='%s\n{\n        type            kqRWallFunction;\n        value           uniform %f;\n}\n'
k_dict['symmetry']='%s\n{\n       type            empty;\n}\n'
k_dict['footer']='}\n'

# formats for the 0/epsilon file
eps_dict={}
eps_dict['header']='''
/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     | Version:  2.0.1                                 |
|   \\  /    A nd           | Web:      www.OpenFOAM.com                      |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       volScalarField;
    location    "0";
    object      epsilon;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

dimensions      [0 2 -3 0 0 0 0];

internalField   uniform %f;

boundaryField
{
'''  #  need initial value of eps
eps_dict['inlet']='%s\n{\n type            fixedValue;\n        value           uniform %f;\n}\n'
eps_dict['outlet']='%s\n{\n        type            zeroGradient;\n}\n'
eps_dict['wall']='%s\n{\n        type            epsilonWallFunction;\n        value           uniform %f;\n}\n'
eps_dict['symmetry']='%s\n{\n       type            empty;\n}\n'
eps_dict['footer']='}\n'

# formats for the 0/nut file
nut_dict={}
nut_dict['header']='''
/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     | Version:  2.0.1                                 |
|   \\  /    A nd           | Web:      www.OpenFOAM.com                      |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       volScalarField;
    location    "0";
    object      nut;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

dimensions      [0 2 -1 0 0 0 0];

internalField   uniform 0;

boundaryField
{
''' 
nut_dict['inlet']='%s\n{\n type            calculated;\n        value           uniform 0;\n}\n'
nut_dict['outlet']='%s\n{\n        type            calculated;\n        value           uniform 0;\n}\n'
nut_dict['wall']='%s\n{\n        type            nutkWallFunction;\n        value           uniform 0;\n}\n'
nut_dict['symmetry']='%s\n{\n       type            empty;\n}\n'
nut_dict['footer']='}\n'

# formats for the 0/nuTilda file
nutilda_dict={}
nutilda_dict['header']='''
/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     | Version:  2.0.1                                 |
|   \\  /    A nd           | Web:      www.OpenFOAM.com                      |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       volScalarField;
    object      nuTilda;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

dimensions      [0 2 -1 0 0 0 0];

internalField   uniform 0;

boundaryField
{
''' 
nutilda_dict['inlet']='%s\n{\n type            fixedValue;\n        value           uniform 0;\n}\n'
nutilda_dict['outlet']='%s\n{\n        type            zeroGradient;\n}\n'
nutilda_dict['wall']='%s\n{\n        type            zeroGradient;\n}\n'
nutilda_dict['symmetry']='%s\n{\n       type            empty;\n}\n'
nutilda_dict['footer']='}\n'

# transport properties, ie file: constant/transportProperties
transport_dict={}
transport_dict['header']='''
/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     | Version:  2.0.1                                 |
|   \\  /    A nd           | Web:      www.OpenFOAM.com                      |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       dictionary;
    location    "constant";
    object      transportProperties;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

transportModel  Newtonian;

nu              nu [ 0 2 -1 0 0 0 0 ] %f;

CrossPowerLawCoeffs
{
    nu0             nu0 [ 0 2 -1 0 0 0 0 ] 1e-06;
    nuInf           nuInf [ 0 2 -1 0 0 0 0 ] 1e-06;
    m               m [ 0 0 1 0 0 0 0 ] 1;
    n               n [ 0 0 0 0 0 0 0 ] 1;
}

BirdCarreauCoeffs
{
    nu0             nu0 [ 0 2 -1 0 0 0 0 ] 1e-06;
    nuInf           nuInf [ 0 2 -1 0 0 0 0 ] 1e-06;
    k               k [ 0 0 1 0 0 0 0 ] 0;
    n               n [ 0 0 0 0 0 0 0 ] 1;
}


// ************************************************************************* //
'''

### MAIN VARS ###
app_vars={'INLET_GRP':'','OUTLET_GRP':'','WALL_GRP':'','SYMM_GRP':'','INLET_UX':10.,'INLET_UY':0.,'INLET_UZ':0.,'INLET_K':0.3,'INLET_EPS':10.,\
          'TRANSPORT_NU':1e-5,'BASEDIR':'','STUDY':'SimpleFoamStudy','UNVMesh':''}


#############
# FUNCTIONS
#############

def generateStudy(vari):
  "Function to generate a basic simplFOAM study"
  bdir=vari["BASEDIR"].strip()
  prjname=vari["STUDY"].strip()
  meshname=vari["UNVMesh"].strip()
  prjdir=os.path.join(bdir,prjname)
  # copy template directory
  os.system('cp -r %s %s'%(template_path,prjdir))
  # copy mesh (for archive)
  os.system('cp %s %s'%(meshname,prjdir))
  # import mesh using ideasUnvToFoam
  os.system('source %s; ideasUnvToFoam -case %s %s'%(env_path,prjdir,meshname))
  # edit constant/polyMesh/boundary to set 'wall' type
  updateBoundary(prjdir,vari)
  # generate constant/transportProperties
  generateTransport(prjdir,vari)
  # generate initial and boundary conditions  
  generateU(prjdir,vari)
  generateP(prjdir,vari)
  generateK(prjdir,vari)
  generateR(prjdir,vari)
  generateEps(prjdir,vari)
  generateNut(prjdir,vari)
  generateNutilda(prjdir,vari)

def updateBoundary(prjdir,vari):
  # read file
  fid=file(os.path.join(prjdir,'constant','polyMesh','boundary'),'r')
  txt=fid.readlines()
  fid.close()
  # update 'wall' groups to type wall
  wallgrps=vari['WALL_GRP'].split()
  for i in range(len(txt)):
    line=txt[i]
    for grp in wallgrps:
      if line.find(grp)>-1:
        txt[i+2]=txt[i+2].replace('patch','wall')
  # update 'symm' groups to type empty
  symmgrps=vari['SYMM_GRP'].split()
  for i in range(len(txt)):
    line=txt[i]
    for grp in symmgrps:
      if line.find(grp)>-1:
        txt[i+2]=txt[i+2].replace('patch','empty')
  fid=file(os.path.join(prjdir,'constant','polyMesh','boundary'),'w')
  fid.writelines(txt)
  fid.close()
   
def generateU(prjdir,vari):
  fid=file(os.path.join(prjdir,'0','U'),'w')
  fid.write(U_dict['header'])
  for grp in vari['INLET_GRP'].split():
    fid.write(U_dict['inlet']%(grp,vari['INLET_UX'],vari['INLET_UY'],vari['INLET_UZ']))
  for grp in vari['OUTLET_GRP'].split():
    fid.write(U_dict['outlet']%(grp))
  for grp in vari['WALL_GRP'].split():
    fid.write(U_dict['wall']%(grp))
  for grp in vari['SYMM_GRP'].split():
    fid.write(U_dict['symmetry']%(grp))
  fid.write(U_dict['footer'])
  fid.close()

def generateP(prjdir,vari):
  fid=file(os.path.join(prjdir,'0','p'),'w')
  fid.write(p_dict['header'])
  for grp in vari['INLET_GRP'].split():
    fid.write(p_dict['inlet']%(grp))
  for grp in vari['OUTLET_GRP'].split():
    fid.write(p_dict['outlet']%(grp))
  for grp in vari['WALL_GRP'].split():
    fid.write(p_dict['wall']%(grp))
  for grp in vari['SYMM_GRP'].split():
    fid.write(p_dict['symmetry']%(grp))
  fid.write(p_dict['footer'])
  fid.close()

def generateR(prjdir,vari):
  fid=file(os.path.join(prjdir,'0','R'),'w')
  fid.write(R_dict['header'])
  for grp in vari['INLET_GRP'].split():
    fid.write(R_dict['inlet']%(grp))
  for grp in vari['OUTLET_GRP'].split():
    fid.write(R_dict['outlet']%(grp))
  for grp in vari['WALL_GRP'].split():
    fid.write(R_dict['wall']%(grp))
  for grp in vari['SYMM_GRP'].split():
    fid.write(R_dict['symmetry']%(grp))
  fid.write(R_dict['footer'])
  fid.close()

def generateK(prjdir,vari):
  fid=file(os.path.join(prjdir,'0','k'),'w')
  fid.write(k_dict['header']%(vari['INLET_K']))
  for grp in vari['INLET_GRP'].split():
    fid.write(k_dict['inlet']%(grp,vari['INLET_K']))
  for grp in vari['OUTLET_GRP'].split():
    fid.write(k_dict['outlet']%(grp))
  for grp in vari['WALL_GRP'].split():
    fid.write(k_dict['wall']%(grp,vari['INLET_K']))
  for grp in vari['SYMM_GRP'].split():
    fid.write(k_dict['symmetry']%(grp))
  fid.write(k_dict['footer'])
  fid.close()

def generateEps(prjdir,vari):
  fid=file(os.path.join(prjdir,'0','epsilon'),'w')
  fid.write(eps_dict['header']%(vari['INLET_EPS']))
  for grp in vari['INLET_GRP'].split():
    fid.write(eps_dict['inlet']%(grp,vari['INLET_EPS']))
  for grp in vari['OUTLET_GRP'].split():
    fid.write(eps_dict['outlet']%(grp))
  for grp in vari['WALL_GRP'].split():
    fid.write(eps_dict['wall']%(grp,vari['INLET_EPS']))
  for grp in vari['SYMM_GRP'].split():
    fid.write(eps_dict['symmetry']%(grp))
  fid.write(eps_dict['footer'])
  fid.close()

def generateNut(prjdir,vari):
  fid=file(os.path.join(prjdir,'0','nut'),'w')
  fid.write(nut_dict['header'])
  for grp in vari['INLET_GRP'].split():
    fid.write(nut_dict['inlet']%(grp))
  for grp in vari['OUTLET_GRP'].split():
    fid.write(nut_dict['outlet']%(grp))
  for grp in vari['WALL_GRP'].split():
    fid.write(nut_dict['wall']%(grp))
  for grp in vari['SYMM_GRP'].split():
    fid.write(nut_dict['symmetry']%(grp))
  fid.write(nut_dict['footer'])
  fid.close()

def generateNutilda(prjdir,vari):
  fid=file(os.path.join(prjdir,'0','nuTilda'),'w')
  fid.write(nutilda_dict['header'])
  for grp in vari['INLET_GRP'].split():
    fid.write(nutilda_dict['inlet']%(grp))
  for grp in vari['OUTLET_GRP'].split():
    fid.write(nutilda_dict['outlet']%(grp))
  for grp in vari['WALL_GRP'].split():
    fid.write(nutilda_dict['wall']%(grp))
  for grp in vari['SYMM_GRP'].split():
    fid.write(nutilda_dict['symmetry']%(grp))
  fid.write(nutilda_dict['footer'])
  fid.close()

def generateTransport(prjdir,vari):
  fid=file(os.path.join(prjdir,'constant','transportProperties'),'w')
  fid.write(transport_dict['header']%(vari['TRANSPORT_NU']))
  fid.close()

########
# GUI
########

class SimpleFoamWizardApp(Frame):
    def createWidgets(self):
        irow=0
        # Study name
        irow=irow+1
        self.prjlbl=Label(self,text="Study Name: ")
        self.prjname=Entry(self,width=40)
        self.prjlbl.grid(row=irow,column=1)
        self.prjname.grid(row=irow,column=2)
        # basedir
        irow=irow+1
        self.basedirlbl=Label(self,text="Base directory: ")
        self.basedirname=Entry(self,width=40)
        self.basedirlbl.grid(row=irow,column=1)
        self.basedirname.grid(row=irow,column=2)
        self.basedirbtn=Button(self,text="...",command=self.selectBaseDir)
        self.basedirbtn.grid(row=irow,column=3)
        # UNV mesh file
        irow=irow+1
        self.meshlbl=Label(self,text="UNV Mesh File: ")
        self.meshname=Entry(self,width=40)
        self.meshlbl.grid(row=irow,column=1)
        self.meshname.grid(row=irow,column=2)
        self.meshbtn=Button(self,text="...",command=self.selectMesh)
        self.meshbtn.grid(row=irow,column=3)
        # INLET_GRP
        irow=irow+1
        self.inletgrplbl=Label(self,text="Inlet Groups: ")
        self.inletgrp=Entry(self,width=40)
        self.inletgrplbl.grid(row=irow,column=1)
        self.inletgrp.grid(row=irow,column=2)
        # OUTLET_GRP
        irow=irow+1
        self.outletgrplbl=Label(self,text="Outlet Groups: ")
        self.outletgrp=Entry(self,width=40)
        self.outletgrplbl.grid(row=irow,column=1)
        self.outletgrp.grid(row=irow,column=2)
        # WALL_GRP
        irow=irow+1
        self.wallgrplbl=Label(self,text="Wall Groups: ")
        self.wallgrp=Entry(self,width=40)
        self.wallgrplbl.grid(row=irow,column=1)
        self.wallgrp.grid(row=irow,column=2)
        # SYMM_GRP
        irow=irow+1
        self.symmgrplbl=Label(self,text="Symm. Groups: ")
        self.symmgrp=Entry(self,width=40)
        self.symmgrplbl.grid(row=irow,column=1)
        self.symmgrp.grid(row=irow,column=2)
        # INLET_UX
        irow=irow+1
        self.inletuxlbl=Label(self,text="Inlet Ux: ")
        self.inletux=Entry(self,width=40)
        self.inletuxlbl.grid(row=irow,column=1)
        self.inletux.grid(row=irow,column=2)
        # INLET_UY
        irow=irow+1
        self.inletuylbl=Label(self,text="Inlet Uy: ")
        self.inletuy=Entry(self,width=40)
        self.inletuylbl.grid(row=irow,column=1)
        self.inletuy.grid(row=irow,column=2)
        # INLET_UZ
        irow=irow+1
        self.inletuzlbl=Label(self,text="Inlet Uz: ")
        self.inletuz=Entry(self,width=40)
        self.inletuzlbl.grid(row=irow,column=1)
        self.inletuz.grid(row=irow,column=2)
        # INLET_K
        irow=irow+1
        self.inletklbl=Label(self,text="Inlet K: ")
        self.inletk=Entry(self,width=40)
        self.inletklbl.grid(row=irow,column=1)
        self.inletk.grid(row=irow,column=2)
        # INLET_EPS
        irow=irow+1
        self.inletepslbl=Label(self,text="Inlet Epsilon: ")
        self.inleteps=Entry(self,width=40)
        self.inletepslbl.grid(row=irow,column=1)
        self.inleteps.grid(row=irow,column=2)
        # TRANSPORT_NU
        irow=irow+1
        self.nulbl=Label(self,text="Nu: ")
        self.nu=Entry(self,width=40)
        self.nulbl.grid(row=irow,column=1)
        self.nu.grid(row=irow,column=2)
        # buttons
        irow=irow+1
        self.closebtn=Button(self,text="Quit",command=self.doquit)
        self.closebtn.grid(row=irow,column=3)
        self.okbtn=Button(self,text="GO",command=self.validate)
        self.okbtn.grid(row=irow,column=1)
                
    def __init__(self,master=None):        
        self.vars=app_vars
        Frame.__init__(self,master)
        self.pack()
        self.createWidgets()

    def doquit(self):
        self.quit()

    def selectBaseDir(self):
        titl="Select a base directory:"
        self.setText(self.basedirname,askdirectory(title=titl))

    def selectMesh(self):
        self.setText(self.meshname,askopenfilename(title="Select a UNV mesh file",filetypes=(("UNV file ","*.unv"),)))

    def getvars(self):
        self.vars["INLET_GRP"]=self.inletgrp.get()
        self.vars["OUTLET_GRP"]=self.outletgrp.get()
        self.vars["WALL_GRP"]=self.wallgrp.get()
        self.vars["SYMM_GRP"]=self.symmgrp.get()
        self.vars["INLET_UX"]=float(self.inletux.get())
        self.vars["INLET_UY"]=float(self.inletuy.get())
        self.vars["INLET_UZ"]=float(self.inletuz.get())
        self.vars["INLET_K"]=float(self.inletk.get())
        self.vars["INLET_EPS"]=float(self.inleteps.get())
        self.vars["TRANSPORT_NU"]=float(self.nu.get())
        self.vars["BASEDIR"]=self.basedirname.get()
        self.vars["UNVMesh"]=self.meshname.get()
        self.vars["STUDY"]=self.prjname.get()
        
    def setText(self,obj,txt):
        obj.delete(0,65535)
        obj.insert(0,txt)

    def copyfile(self,filein,fileout):
        fd1=open(filein,"r")
        fd2=open(fileout,"w")
        fd2.write(fd1.read())
        fd1.close()
        fd2.close()        

    def validate(self):
        # retrieve variables
        self.getvars()
        bdir=self.vars["BASEDIR"].strip()
        prjname=self.vars["STUDY"].strip().upper()
        meshname=self.vars["UNVMesh"].strip()
        prjdir=os.path.join(bdir,prjname)
        if os.path.exists(prjdir):
            showerror(title="Error",message="Error, Study folder allready exists in base directory")
        else:
            #run script to create SimpleFOAM case
            generateStudy(self.vars)
	    showinfo(title="Success",message=("The new SimpleFOAM study has being created. Please check the generated files and then run 'simpleFoam' to start the solver"))
	    self.doquit()

########
# MAIN
########
app=SimpleFoamWizardApp()
top=app.master
top.wm_title("SimpleFOAM CFD Study Wizard")
app.mainloop()
